//*****************main.c***************************************
// Test of bit-mapped graphics on the AGM1264F MODULE
// Jonathan W. Valvano 11/20/09 

//  This example accompanies the books
//   "Embedded Microcomputer Systems: Real Time Interfacing",
//         Engineering, copyright (c) 2006,
//   "Introduction to Embedded Microcomputer Systems: 
//    Motorola 6811 and 6812 Simulation", Thomson, copyright (c) 2002

// Copyright 2007 by Jonathan W. Valvano, valvano@mail.utexas.edu 
//    You may use, edit, run or distribute this file 
//    as long as the above copyright notice remains 

// Hardware:
// gnd    =  1- AGM1264F ground
// +5V    =  2- AGM1264F Vcc (with 0.1uF cap to ground)
// pot    =  3- AGM1264F Vo (center pin of 10k pot)
// PAD4   =  4- AGM1264F D/I (0 for command, 1 for data)
// gnd    =  5- AGM1264F R/W (blind cycle synchronization)
// PT3    =  6- AGM1264F E   (1 to latch in data/command)
// PAD0   =  7- AGM1264F DB0
// PAD1   =  8- AGM1264F DB1
// PAD2   =  9- AGM1264F DB2
// PAD3   = 10- AGM1264F DB3
// PT4    = 11- AGM1264F DB4
// PT5    = 12- AGM1264F DB5
// PT6    = 13- AGM1264F DB6
// PT7    = 14- AGM1264F DB7
// PM0    = 15- AGM1264F CS1 (controls left half of LCD)
// PM1    = 16- AGM1264F CS2 (controls right half of LCD)
// +5V    = 17- AGM1264F RES (reset)
// pot    = 18- ADM1264F Vee (-10V)
// 10k pot from pin 18 to ground, with center to pin 3
// references   http://www.azdisplays.com/prod/g1264f.php
// sample code  http://www.azdisplays.com/PDF/agm1264f_code.pdf
// data sheet   http://www.azdisplays.com/PDF/agm1264f.pdf
//******************************************************************
#include <hidef.h>         /* common defines and macros */
#include <mc9s12c32.h>     /* derivative information */

#include "PLL.h"
#include "LCDG.h"
#include "Timer.h"

#pragma LINK_INFO DERIVATIVE "mc9s12c32"

//********GetData***************
// simulated ramp wave, 10ms delay
// inputs: none
// outputs: -500 to 500 ramp
short Data;        // made-up samples
short GetData(void){
  Timer_Wait1ms(10);
  Data +=2;
  if(Data>500) Data=-500;
  return Data;
}
  
void main(void) {  
unsigned short start; // beginning of 4 second sweep 
unsigned short time;  // relative time along sweep in msec
  DDRT |= 0x03;       // PT1, PT0 are debugging leds
  PLL_Init();
  LCD_Init();
  EnableInterrupts;
  
  LCD_Test();  // to run this, set TEST=1 inside LCDG.c, single step through
  LCD_Clear(0);

  LCD_GoTo(1,1);    // top line
  LCD_OutString("AGM1264 LCD graphics");
  LCD_GoTo(8,1);    // bottom line
  LCD_OutString(" U Texas, Valvano ");
  LCD_OutChar(127); // UT symbol
  time = 0;
  start = 0;
  Data = -500; // made-up samples
  LCD_PlotYaxis(-5, 0, 5, 0, "Voltage");
  LCD_PlotClear(-500, 500, 4); //4*100=400 points/sweep
  LCD_PlotXaxis(start, start+4, 0, "Time (sec)");
  for(;;){
    if(time==4000){   // 4 seconds/sweep
      PTT ^= 0x01;    // debugging
      start = start+4;
      if(start>900) start=0;
      LCD_PlotXaxis(start, start+4, 0, "Time (sec)");
      time = 0;
    }
    LCD_Plot(GetData());
    time += 10; // time in msec
  }
}


