//*****************main.c***************************************
// 2/13/07
// Test of bit-mapped graphics on the AGM1264F MODULE
// Hardware:
// PAD3-0 = AGM1264F DB3-DB0
// PT7-4  = AGM1264F DB7-DB4
// PM2    = AGM1264F D/I
// PM3    = AGM1264F CS1 (controls left half of LCD)
// PM4    = AGM1264F E
// PM5    = AGM1264F CS2 (controls right half of LCD)
// +5V    = AGM1264F RES 
// PM1    = AGM1264F R/W (used to read busy flag)
// references   http://www.azdisplays.com/prod/g1264f.php
// sample code  http://www.azdisplays.com/PDF/agm1264f_code.pdf
// data sheet   http://www.azdisplays.com/PDF/agm1264f.pdf
//******************************************************************
#include <hidef.h>         /* common defines and macros */
#include <mc9s12c32.h>     /* derivative information */

#include "PLL.h"

#pragma LINK_INFO DERIVATIVE "mc9s12c32"


#define T1usec 24
#define T4usec (T1usec*4)
//---------------------Timer_Init---------------------
// activate TCNT at 24 MHz
// inputs: none
// outputs: none
void Timer_Init(void){
  asm sei      // make ritual atomic 
  TIOS  |= 0x08;  // activate TC3 as output compare
  TSCR1 = 0x80;   // Enable TCNT, 24MHz E clock
  TSCR2 = 0x00;   // divide by 1 TCNT prescale, TOI disarm
  PACTL = 0;      // timer prescale used for TCNT
/* Bottom three bits of TSCR2 (PR2,PR1,PR0) determine TCNT period
    divide  FastMode(24MHz)    Slow Mode (4MHz)
000   1     42ns  TOF  2.73ms  250ns TOF 16.384ms
001   2     84ns  TOF  5.46ms  500ns TOF 32.768ms 
010   4    167ns  TOF  10.9ms    1us TOF 65.536ms   
011   8    333ns  TOF  21.8ms 	 2us TOF 131.072ms
100  16    667ns  TOF  43.7ms 	 4us TOF 262.144ns
101  32   1.33us  TOF  87.4ms		 8us TOF 524.288ms
110  64   2.67us  TOF 174.8ms   16us TOF 1.048576s
111 128   5.33us  TOF 349.5ms   32us TOF 2.097152s */ 
}

//---------Timer_Wait------------------
// fixed time delay
// inputs: time to wait in 42ns cycles
// outputs: none
void Timer_Wait(unsigned short delay){ 
unsigned short startTime;
  startTime = TCNT;
  while((TCNT-startTime) <= delay){} 
}
//---------Timer_Wait1ms------------------
// fixed time delay
// inputs: time to wait in ms
// outputs: none
// 24000 cycles equals 1ms
void Timer_Wait1ms(unsigned short delay){ 
  for(;delay>0;delay--){
    Timer_Wait(24000);
  }
}

//---------Timer_Wait10ms------------------
// fixed time delay
// inputs: time to wait in 10ms
// outputs: none
void Timer_Wait10ms(unsigned short delay){
  for(;delay>0;delay--){
    Timer_Wait1ms(10);
  }
}
// digits are drawn at the top of the byte
const unsigned char TinyFont[10*3]={  // 3 wide by 5 tall
  14,17,14,   // 0
  18,31,16,   // 1
  25,21,23,   // 2
  17,21,31,   // 3
  7,4,31,     // 4
  23,21,9,    // 5
  31,21,28,   // 6
  1,1,31,     // 7
  31,21,31,   // 8
  7,21,31     // 9
};
// digits are drawn one pixel from the top 
const unsigned char TinyFont1[10*3]={  // 3 wide by 5 tall
  2*14,2*17,2*14,   // 0
  2*18,2*31,2*16,   // 1
  2*25,2*21,2*23,   // 2
  2*17,2*21,2*31,   // 3
  2*7,2*4,2*31,     // 4
  2*23,2*21,2*9,    // 5
  2*31,2*21,2*28,   // 6
  2*1,2*1,2*31,     // 7
  2*31,2*21,2*31,   // 8
  2*7,2*21,2*31     // 9
};
// two pixels from the top or one pixel from bottom 
const unsigned char TinyFont2[10*3]={  // 3 wide by 5 tall
  4*14,4*17,4*14,   // 0
  4*18,4*31,4*16,   // 1
  4*25,4*21,4*23,   // 2
  4*17,4*21,4*31,   // 3
  4*7,4*4,4*31,     // 4
  4*23,4*21,4*9,    // 5
  4*31,4*21,4*28,   // 6
  4*1,4*1,4*31,     // 7
  4*31,4*21,4*31,   // 8
  4*7,4*21,4*31     // 9
};
// three pixels from the top or at the bottom 
const unsigned char TinyFont3[10*3]={  // 3 wide by 5 tall
  8*14,8*17,8*14,   // 0
  8*18,8*31,8*16,   // 1
  8*25,8*21,8*23,   // 2
  8*17,8*21,8*31,   // 3
  8*7,8*4,8*31,     // 4
  8*23,8*21,8*9,    // 5
  8*31,8*21,8*28,   // 6
  8*1,8*1,8*31,     // 7
  8*31,8*21,8*31,   // 8
  8*7,8*21,8*31     // 9
};
// 5 wide by 7 tall font
const unsigned char Font[95*5]={   // no numbers with bit7=1
  0,0,0,0,0,           // 32  space
  0,0,95,0,0,          // 33  !
  0,7,0,7,0,           // 34  "
  20,127,20,127,20,    // 35  #
  36,42,127,42,18,     // 36  $
  35,19,8,100,98,      // 37  %
  54,73,85,34,80,      // 38  &
  0,5,3,0,0,           // 39  quote
  0,28,34,65,0,        // 40  (
  0,65,34,28,0,        // 41  )
  20,8,62,8,20,        // 42  *
  8,8,62,8,8,          // 43  plus
  0,80,48,0,0,         // 44  ,
  8,8,8,8,8,           // 45  minus
  0,112,112,112,0,     // 46  .
  32,16,8,4,2,         // 47  /
  62,81,73,69,62,      // 48  0
  0,66,127,64,0,       // 49  1
  66,97,81,73,70,      // 50  2
  33,65,69,75,49,      // 51  3
  24,20,18,127,16,     // 52  4
  39,69,69,69,57,      // 53  5
  60,74,73,73,48,      // 54  6
  3,1,113,9,7,         // 55  7
  54,73,73,73,54,      // 56  8
  6,73,73,41,30,       // 57  9
  0,54,54,0,0,         // 58  :
  0,86,54,0,0,         // 59  ;
  8,20,34,65,0,        // 60  <
  20,20,20,20,20,      // 61  equals
  65,34,20,8,0,        // 62  >
  2,1,81,9,6,          // 63  ?
  50,73,121,65,62,     // 64  @
  126,17,17,17,126,    // 65  A
  127,73,73,73,54,     // 66  B
  62,65,65,65,34,      // 67  C
  127,65,65,65,62,     // 68  D
  127,73,73,73,65,     // 69  E
  127,9,9,9,1,         // 70  F
  62,65,73,73,122,     // 71  G
  127,8,8,8,127,       // 72  H
  65,65,127,65,65,     // 73  I
  32,64,65,63,1,       // 74  J
  127,8,20,34,65,      // 75  K
  127,64,64,64,64,     // 76  L
  127,2,12,2,127,      // 77  M
  127,6,24,96,127,     // 78  N
  62,65,65,65,62,      // 79  O
  127,9,9,9,6,         // 80  P
  62,65,81,33,94,      // 81  Q
  127,9,25,41,70,      // 82  R
  70,73,73,73,49,      // 83  S
  1,1,127,1,1,         // 84  T
  63,64,64,64,63,      // 85  U
  31,32,64,32,31,      // 86  V
  63,64,56,64,63,      // 87  W
  99,20,8,20,99,       // 88  X
  7,8,112,8,7,         // 89  Y
  97,81,73,69,67,      // 90  Z
  0,127,65,65,0,       // 91  [
  2,4,8,16,32,         // 92  back slash
  0,65,65,127,0,       // 93  ]
  4,2,1,2,4,           // 94  ^
  64,64,64,64,64,      // 95  _
  0,1,2,4,0,           // 96  quote
  32,84,84,84,120,     // 97  a
  127,72,68,68,56,     // 98  b
  56,68,68,68,32,      // 99  c
  56,68,68,72,127,     // 100  d
  56,84,84,84,24,      // 101  e
  8,126,9,1,2,         // 102  f
  8,84,84,84,60,       // 103  g   
  127,8,4,4,120,       // 104  h
  0,72,125,64,0,       // 105  i
   32,64,68,61,0,      // 106  j  
  127,16,40,68,0,      // 107  k
  0,65,127,64,0,       // 108  l
  124,4,24,4,120,      // 109  m
  124,8,4,4,120,       // 110  n
  56,68,68,68,56,      // 111  o
  124,20,20,20,8,      // 112  p      
  12,18,18,20,126,     // 113  q   
  124,8,4,4,8,         // 114  r
  72,84,84,84,36,      // 115  s
  4,63,68,64,32,       // 116  t
  60,64,64,32,124,     // 117  u
  28,32,64,32,28,      // 118  v
  60,64,48,64,60,      // 119  w
  68,40,16,40,68,      // 120  x
  12,80,80,80,60,      // 121  y    
  68,100,84,76,68,     // 122  z
  0,65,54,8,0,         // 123  }
  0,0,127,0,0,         // 124  |
  0,8,54,65,0,         // 125  {
  8,4,8,16,8};         // 126  ~


#define E PTM_PTM4
#define DI PTM_PTM2
#define RW PTM_PTM1
#define CS2 PTM_PTM5
#define CS1 PTM_PTM3
static unsigned short OpenFlag=0;

// ********* lcdStatus***********
// Read status from AGM1264F 128-bit by 64-bit graphics display
// Inputs: none
// Outputs: status byte (bit 7 is BUSY=1, Ready=0)
unsigned char lcdStatus(void){
unsigned char status;
  RW = 1;        // read mode 
  DI = 0;        // D/I=0, status read
  Timer_Wait(T1usec);
  E = 1;         // E pulse width > 450ns
  Timer_Wait(T1usec);
  status = PTT&0xF0;   // bits 7-4
  status += PTAD&0x0F; // bits 3-0
  E = 0;         // falling edge latch, setup time 200ns
  DI = 1;        // D/I=1 default state is data
  RW = 0;        // write mode 
  return status;
}

// ********* lcdCmdBlind***********
// Output command to AGM1264F 128-bit by 64-bit graphics display
// Blind wait
// Inputs: 8-bit instruction
// Outputs: none
void lcdCmdBlind(unsigned char instruction){
  // R/W=0, write mode  default, R/W=0 always
  // normally D/I will be left at D/I=1 for data
  DI = 0;        // D/I=0, COMMAND WRITE
  Timer_Wait(T1usec);
  E = 1;         // E pulse width > 450ns
  PTT = (PTT&0x0F)+(instruction&0xF0);   // bits 7-4
  PTAD = (PTAD&0xF0)+(instruction&0x0F); // bits 3-0
  Timer_Wait(T1usec);
  E = 0;         // falling edge latch, setup time 200ns
  DI = 1;        // D/I=1 default state is data
  Timer_Wait(T4usec);
} 

// ********* lcdCmd***********
// Output command to AGM1264F 128-bit by 64-bit graphics display
// Inputs: 8-bit instruction
// Outputs: none
void lcdCmd(unsigned char instruction){
//  while(lcdStatus()&0x80){
//  }
  // R/W=0, write mode  default, R/W=0 always
  // normally D/I will be left at D/I=1 for data
  DI = 0;        // D/I=0, COMMAND WRITE
  Timer_Wait(T1usec);
  E = 1;         // E pulse width > 450ns
  PTT = (PTT&0x0F)+(instruction&0xF0);   // bits 7-4
  PTAD = (PTAD&0xF0)+(instruction&0x0F); // bits 3-0
  Timer_Wait(T1usec);
  E = 0;         // falling edge latch, setup time 200ns
  DI = 1;        // D/I=1 default state is data
  Timer_Wait(T4usec);
} 

// ********* lcdData***********
// Output data to AGM1264F 128-bit by 64-bit graphics display
// Inputs: 8-bit data
// Outputs: none
void lcdData(unsigned char data){
//  while(lcdStatus()&0x80){
//  }
  // R/W=0, write mode  default, R/W=0 always
  // normally D/I will be left at D/I=1 for data
  E = 1;         // E pulse width > 450ns
  PTT = (PTT&0x0F)+(data&0xF0);   // bits 7-4
  PTAD = (PTAD&0xF0)+(data&0x0F); // bits 3-0
  Timer_Wait(T1usec);
  E = 0;         // falling edge latch, setup time 200ns
  Timer_Wait(T4usec);
}    

// ********* LCD_Init***********
// Initialize AGM1264F 128-bit by 64-bit graphics display
// PAD3-0 = AGM1264F DB3-DB0
// PT7-4  = AGM1264F DB7-DB4
// PM2    = AGM1264F D/I
// PM3    = AGM1264F CS1 (controls left half of LCD)
// PM4    = AGM1264F E
// PM5    = AGM1264F CS2 (controls right half of LCD)
// PM1    = AGM1264F R/W
// Inputs: none
// Outputs: none
void LCD_Init(void){
  Timer_Init();
  DDRT |= 0xF0;    // PT7-PT4 outputs to DB7-DB4
  DDRAD |= 0x0F;   // PAD3-PAD0 outputs to DB3-DB0
  ATDDIEN |= 0x0F; // enable AD digital output drivers
  CS2 = 1;         // talk to both LCD controllers
  CS1 = 1;
  DI = 1;          // default mode is data 
  E = 0;           // inactive
  RW = 0;          // default mode is write
  DDRM |= 0x3E;    // 5 control signals are output  
  lcdCmdBlind(0x3F);    // display=ON
  lcdCmdBlind(0xB8);    // Page address (0 to 7) is 0
  lcdCmdBlind(0x40);    // Column address (0 to 63) is 0
  lcdCmdBlind(0xC0);    // Y=0 is at top
  OpenFlag = 1;     // device open
}


// ********* LCD_Clear***********
// Clear the entire 1024 byte (8192 bit) image on the
//    AGM1264F 128-bit by 64-bit graphics display
// Inputs: value to set all bytes to
// Outputs: none
void LCD_Clear(unsigned char data){
unsigned char page;
  int i;
  if(OpenFlag == 0) return;
  for(page = 0xB8; page< 0xB8+8; page++){ // pages 0 to 7
    CS2 = 1;          // right enable
    CS1 = 0;         
    lcdCmd(page);    // Page address (0 to 7) 
    lcdCmd(0x40);    // Column = 0
    for(i=64; i>0; i--){
      lcdData(data);   // copy one byte to right side
    }
    CS2 = 0;          
    CS1 = 1;         // left enable
    lcdCmd(page);    // Page address (0 to 7) 
    lcdCmd(0x40);    // Column = 0
    for(i=64; i>0; i--){
      lcdData(data);   // copy one byte to left side
    }
  }
}


// ********* LCD_VTest***********
// Clear the vertical lines image on the
//    AGM1264F 128-bit by 64-bit graphics display
// Inputs: none
// Outputs: none
void LCD_VTest(void){
unsigned char page;
  int i;
  for(page = 0xB8; page< 0xB8+8; page++){ // pages 0 to 7
    CS2 = 1;          // right enable
    CS1 = 0;         
    lcdCmd(page);    // Page address (0 to 7) 
    lcdCmd(0x40);    // Column = 0
    for(i=32; i>0; i--){
      lcdData(0x00);   // copy one byte to right
      lcdData(0xFF);   // copy one byte to right
    }
    CS2 = 0;          
    CS1 = 1;         // left enable
    lcdCmd(page);    // Page address (0 to 7) 
    lcdCmd(0x40);    // Column = 0
    for(i=32; i>0; i--){
      lcdData(0x00);   // copy one byte to left
      lcdData(0xFF);   // copy one byte to left
    }
  }
}

// ********* LCD_DrawImage***********
// Draw an entire 1024 byte (8192 bit) image on the
//    AGM1264F 128-bit by 64-bit graphics display
// Inputs: pointer to 1024 bytes of data
// Outputs: none
void LCD_DrawImage(const unsigned char *pt){
unsigned char page;
  int i;
  if(OpenFlag == 0) return;
  for(page = 0xB8; page< 0xB8+8; page++){ // pages 0 to 7
    CS2 = 0;
    CS1 = 1;         // left enable
    lcdCmd(page);    // Page address (0 to 7) 
    lcdCmd(0x40);    // Column = 0
    for(i=64; i>0; i--){
      lcdData(*pt);  // copy one byte to left
      pt++;
    }
    
    CS1 = 0;
    CS2 = 1;          // right enable
    lcdCmd(page);     // Page address (0 to 7) 
    lcdCmd(0x40);     // Column = 0
    for(i=64; i>0; i--){
      lcdData(*pt);   // copy one byte to right
      pt++;
    }
  }
}
// ********* LCD_OutChar***********
// Output ASCII character on the
//    AGM1264F 128-bit by 64-bit graphics display
// Inputs: 7-bit ASCII to display
// Outputs: none
// letter must be between 32 and 126 inclusive
unsigned char Column1=0x41; // column position
unsigned char bLeft1=1;
unsigned char bRight1=0;
unsigned char Page=0xB8;
unsigned char bDown=0;    // true if want font shifted down
unsigned char TinyDown=0; // 0 no shift, 1,2,3 shift down 

void LCD_OutChar(unsigned char letter){
unsigned short i,cnt;
  if(OpenFlag == 0) return;
// page 0 is 0xB8, varies from 0xB7 to 0xBF
  if(letter<32) return;
  if(letter>126) return;
  i = 5*(letter-32); // index into font table
  CS2 = bRight1;   // right enable
  CS1 = bLeft1;    // left enable
  lcdCmd(Page);    // Page address 0 to 7 
  lcdCmd(Column1); // Column = 0
  for(cnt=5; cnt>0; cnt--){
    if(bDown){
      lcdData(Font[i]<<1);  // copy one byte, shifted down 
    } else{
      lcdData(Font[i]);  // copy one byte 
    }
    i++;
    Column1++;
    if(bLeft1&&(Column1==0x80)){
      Column1 = 0x40;
      bLeft1 = 0;  
      bRight1 = 1;     // switch to right side
      CS2 = bRight1;   // right enable
      CS1 = bLeft1;    // left enable
      lcdCmd(Page);    // Page address 0 to 7) 
      lcdCmd(Column1); // Column = 0
    }
    if(bRight1&&(Column1==0x7F)){
      Column1 = 0x41;
      bLeft1 = 1;  
      bRight1 = 0;     // switch to left side
      CS2 = bRight1;   // right enable
      CS1 = bLeft1;    // left enable
      lcdCmd(Page);    // Page address 0 to 7) 
      lcdCmd(Column1); // Column = 0
    }
  }
  lcdData(0);  // inter-character space copy one byte 
  Column1++;
  if(bLeft1&&(Column1==0x80)){
    Column1 = 0x40;
    bLeft1 = 0;  
    bRight1 = 1;     // switch to right side
    CS2 = bRight1;   // right enable
    CS1 = bLeft1;    // left enable
    lcdCmd(Page);    // Page address 0 to 7) 
    lcdCmd(Column1); // Column = 0
  }
  if(bRight1&&(Column1==0x7F)){
    Column1 = 0x41;
    bLeft1 = 1;  
    bRight1 = 0;     // switch to left side
    CS2 = bRight1;   // right enable
    CS1 = bLeft1;    // left enable
    lcdCmd(Page);    // Page address 0 to 7) 
    lcdCmd(Column1); // Column = 0
  }
}  


  
//---------------------LCD_OutString--------------
// Display String
// Input: pointer to NULL-terminationed ASCII string 
// Output: none
void LCD_OutString(char *pt){ 
  if(OpenFlag==0){
    return;  // not open
  }
  while(*pt){
    LCD_OutChar((unsigned char)*pt);
    pt++;
  }
}
  


//-----------------------LCD_OutDec-----------------------
// Output a 16-bit number in unsigned decimal format
// Input: 16-bit unsigned number 
// Output: none
// fixed size 3 digits of output, right justified 
void LCD_OutDec(unsigned short n){
  if(OpenFlag==0){
    return;  // not open
  }
  if(n < 10){
    LCD_OutString("    ");
    LCD_OutChar(n+'0'); /* n is between 0 and 9 */
  } else if(n<100){
    LCD_OutString("   ");
    LCD_OutChar(n/10+'0'); /* tens digit */
    LCD_OutChar(n%10+'0'); /* ones digit */
  } else if(n<1000){
      LCD_OutString("  ");
      LCD_OutChar(n/100+'0'); /* hundreds digit */
      n = n%100;
      LCD_OutChar(n/10+'0'); /* tens digit */
      LCD_OutChar(n%10+'0'); /* ones digit */
    }
    else if(n<10000){
      LCD_OutChar(' ');  
      LCD_OutChar(n/1000+'0'); /* thousands digit */
      n = n%1000;
      LCD_OutChar(n/100+'0'); /* hundreds digit */
      n = n%100;
      LCD_OutChar(n/10+'0'); /* tens digit */
      LCD_OutChar(n%10+'0'); /* ones digit */
    }
    else {
      LCD_OutChar(n/10000+'0'); /* ten-thousands digit */
      n = n%10000;
      LCD_OutChar(n/1000+'0'); /* thousands digit */
      n = n%1000;
      LCD_OutChar(n/100+'0'); /* hundreds digit */
      n = n%100;
      LCD_OutChar(n/10+'0'); /* tens digit */
      LCD_OutChar(n%10+'0'); /* ones digit */
    }
}

//-----------------------LCD_OutFix1-----------------------
// Output a 16-bit number in unsigned decimal fixed-point
// with resolution = 0.1 
// Input: 16-bit unsigned number 
// Output: none
// fixed size is 6 characters of output, right justified
// if input is 12345, then display is 1234.5 
void LCD_OutFix1(unsigned short n){
  if(OpenFlag==0){
    return;  // not open
  }
  if(n < 10){
    LCD_OutString("   0.");
    LCD_OutChar(n+'0'); /* n is between 0 and 9 */
  } else if(n<100){
    LCD_OutString("   ");
    LCD_OutChar(n/10+'0'); /* tens digit */
    LCD_OutChar('.');      /* decimal point */
    LCD_OutChar(n%10+'0'); /* ones digit */
  } else if(n<1000){
      LCD_OutString("  ");
      LCD_OutChar(n/100+'0'); /* hundreds digit */
      n = n%100;
      LCD_OutChar(n/10+'0'); /* tens digit */
      LCD_OutChar('.');      /* decimal point */
      LCD_OutChar(n%10+'0'); /* ones digit */
    }
    else if(n<10000){
      LCD_OutChar(' ');  
      LCD_OutChar(n/1000+'0'); /* thousands digit */
      n = n%1000;
      LCD_OutChar(n/100+'0'); /* hundreds digit */
      n = n%100;
      LCD_OutChar(n/10+'0'); /* tens digit */
      LCD_OutChar('.');      /* decimal point */
      LCD_OutChar(n%10+'0'); /* ones digit */
    }
    else {
      LCD_OutChar(n/10000+'0'); /* ten-thousands digit */
      n = n%10000;
      LCD_OutChar(n/1000+'0'); /* thousands digit */
      n = n%1000;
      LCD_OutChar(n/100+'0'); /* hundreds digit */
      n = n%100;
      LCD_OutChar(n/10+'0'); /* tens digit */
      LCD_OutChar('.');      /* decimal point */
      LCD_OutChar(n%10+'0'); /* ones digit */
    }
}
//-----------------------LCD_OutFix2-----------------------
// Output a 16-bit number in unsigned decimal fixed-point
// with resolution = 0.01 
// Input: 16-bit unsigned number 
// Output: none
// fixed size is 6 characters of output, right justified
// if input is 12345, then display is 123.45 
void LCD_OutFix2(unsigned short n){
  if(OpenFlag==0){
    return;  // not open
  }
  if(n < 10){
    LCD_OutString("  0.0");
    LCD_OutChar(n+'0'); /* n is between 0 and 9 */
  } else if(n<100){
    LCD_OutString("  0.");
    LCD_OutChar(n/10+'0'); /* tens digit */
    LCD_OutChar(n%10+'0'); /* ones digit */
  } else if(n<1000){
      LCD_OutString("  ");
      LCD_OutChar(n/100+'0'); /* hundreds digit */
      n = n%100;
      LCD_OutChar('.');      /* decimal point */
      LCD_OutChar(n/10+'0'); /* tens digit */
      LCD_OutChar(n%10+'0'); /* ones digit */
    }
    else if(n<10000){
      LCD_OutChar(' ');  
      LCD_OutChar(n/1000+'0'); /* thousands digit */
      n = n%1000;
      LCD_OutChar(n/100+'0'); /* hundreds digit */
      n = n%100;
      LCD_OutChar('.');      /* decimal point */
      LCD_OutChar(n/10+'0'); /* tens digit */
      LCD_OutChar(n%10+'0'); /* ones digit */
    }
    else {
      LCD_OutChar(n/10000+'0'); /* ten-thousands digit */
      n = n%10000;
      LCD_OutChar(n/1000+'0'); /* thousands digit */
      n = n%1000;
      LCD_OutChar(n/100+'0'); /* hundreds digit */
      n = n%100;
      LCD_OutChar('.');      /* decimal point */
      LCD_OutChar(n/10+'0'); /* tens digit */
      LCD_OutChar(n%10+'0'); /* ones digit */
    }
}

//-----------------------LCD_OutFix3-----------------------
// Output a 16-bit number in unsigned decimal fixed-point
// with resolution = 0.001 
// Input: 16-bit unsigned number 
// Output: none
// fixed size is 6 characters of output, right justified
// if input is 12345, then display is 12.345 
void LCD_OutFix3(unsigned short n){
  if(OpenFlag==0){
    return;  // not open
  }
  if(n < 10){
    LCD_OutString(" 0.00");
    LCD_OutChar(n+'0');  /* n is between 0 and 9 */
  } else if(n<100){
    LCD_OutString(" 0.0");
    LCD_OutChar(n/10+'0'); /* tens digit */
    LCD_OutChar(n%10+'0'); /* ones digit */
  } else if(n<1000){
      LCD_OutString(" 0.");
      LCD_OutChar(n/100+'0'); /* hundreds digit */
      n = n%100;
      LCD_OutChar(n/10+'0'); /* tens digit */
      LCD_OutChar(n%10+'0'); /* ones digit */
    }
    else if(n<10000){
      LCD_OutChar(' ');  
      LCD_OutChar(n/1000+'0'); /* thousands digit */
      LCD_OutChar('.');      /* decimal point */
      n = n%1000;
      LCD_OutChar(n/100+'0'); /* hundreds digit */
      n = n%100;
      LCD_OutChar(n/10+'0'); /* tens digit */
      LCD_OutChar(n%10+'0'); /* ones digit */
    }
    else {
      LCD_OutChar(n/10000+'0'); /* ten-thousands digit */
      n = n%10000;
      LCD_OutChar(n/1000+'0'); /* thousands digit */
      LCD_OutChar('.');      /* decimal point */
      n = n%1000;
      LCD_OutChar(n/100+'0'); /* hundreds digit */
      n = n%100;
      LCD_OutChar(n/10+'0'); /* tens digit */
      LCD_OutChar(n%10+'0'); /* ones digit */
    }
}

//-----------------------LCD_GoTo-----------------------
// Move cursor
// Input: line number is 1 to 8, column from 1 to 20 
// Output: none
// errors: it will check for legal address
void LCD_GoTo(int line, int column){
  if(OpenFlag==0){
    return;  // not open
  }
  if((line<1) || (line>8)) return;
  if((column<1) || (column>20)) return;
  if(line<5){
    bDown = 0;    // normal position on lines 1,2,3,4
  } else{
    bDown = 0xFF; // shifted down on lines 5,6,7,8
  }
  Page = 0xB8+line-1; // 0xB8 to 0xBF
  if(column<11){
    Column1 = 59+6*column; //0x41+6*(column-1);
    bLeft1 = 1;  
    bRight1 = 0;     // on left side
  } else{
    Column1 = 1+6*column; //0x43+6*(column-11);
    bLeft1 = 0;  
    bRight1 = 1;     // on right side
  }
}
 
#define PLOT_PIXEL 4
// number of data points drawn into the same X-axis pixel
// 4 means it takes 400 calls to LCD_Plot to draw one sweep
unsigned short SubCount;
unsigned short Xvalue; // ranges from 0 to 99
 


unsigned short XaxisResolution;
// X axis numbers are  0 to 999
// 65535 means no numbers are displayed
// number->       0    9    99   999
// 0  shown as   0.    9.   99.  999.
// 1  shown as  0.0   0.9   9.9  99.9
// 2  shown as 0.00  0.09  0.99  9.99
// 3  shown as .000  .009  .099  .999
unsigned short Xmax,Xmin; // two numbers to put on Xaxis
char Xlabel[14];          // maximum of 13 characters


//---------lcdTinyOutDigit-------------------------
// display tiny digits
// input: digit from 0 to 9 
// output: none
// no error checking
void lcdTinyOutDigit(unsigned short digit){
unsigned char *pt;
  if(TinyDown==0){
    pt = (unsigned char *) &TinyFont[digit*3];
  } else if(TinyDown==1){
    pt = (unsigned char *) &TinyFont1[digit*3];
  } else if(TinyDown==2){
    pt = (unsigned char *) &TinyFont2[digit*3];
  } else {
    pt = (unsigned char *) &TinyFont3[digit*3];
  }
  lcdData(*pt++);  // copy three bytes 
  lcdData(*pt++);   
  lcdData(*pt++);
  lcdData(0);      // intercharacter space   
}
//---------lcdTinyOutPoint-------------------------
// display tiny decimal point
// input: none
// output: none
void lcdTinyOutPoint(void){
  if(TinyDown==0){
    lcdData(16);
  } else if(TinyDown==1){
    lcdData(32);
  } else if(TinyDown==2){
    lcdData(64);
  } else {
    lcdData(128);
  }
  lcdData(0);      // intercharacter space   
}
//---------lcdTinyOutSpace-------------------------
// display space
// input: none
// output: none
void lcdTinyOutSpace(void){
  lcdData(0);      // intercharacter space   
  lcdData(0);      // intercharacter space   
  lcdData(0);      // intercharacter space   
  lcdData(0);      // intercharacter space   
}

//---------lcdTinyOutMinus-------------------------
// display tiny minus sign
// input: none
// output: none
void lcdTinyOutMinus(void){
  if(TinyDown==0){
    lcdData(4);
    lcdData(4);
  } else if(TinyDown==1){
    lcdData(8);
    lcdData(8);
  } else if(TinyDown==2){
    lcdData(16);
    lcdData(16);
  } else {
    lcdData(32);
    lcdData(32);
  }
  lcdData(0);      // intercharacter space   
}

//---------lcdTinyOutPlus-------------------------
// display a little space instead of plus sign
// input: none
// output: none
void lcdTinyOutPlus(void){
  lcdData(0);      // intercharacter space   
  lcdData(0);      // intercharacter space   
  lcdData(0);      // intercharacter space   
}

//------lcdTinyOutFix------------------------
// display 4 character tiny number
// input: num 0 to 999
// output:none
// no error checking
void lcdTinyOutFix(unsigned short num){
  if(XaxisResolution == 0){      // shown as   0.    9.   99.  999.
    if(num < 10){
      lcdTinyOutSpace(); 
      lcdTinyOutSpace();
    } else if(num < 100){
      lcdTinyOutSpace();
      lcdTinyOutDigit(num/10);   // tens digit 1 to 9
    } else{
      lcdTinyOutDigit(num/100);  // hundreds digit 1 to 9
			num = num%100;             // 0 to 99
      lcdTinyOutDigit(num/10);   // tens digit 1 to 9
    }
    lcdTinyOutDigit(num%10);     // ones digit 1 to 9
    lcdTinyOutPoint();
  }else if(XaxisResolution == 1){// shown as  0.0   0.9   9.9  99.9
    if(num < 100){
      lcdTinyOutSpace();
    } else{
      lcdTinyOutDigit(num/100);  // hundreds digit 1 to 9
			num = num%100;             // 0 to 99
    }
    lcdTinyOutDigit(num/10);     // tens digit 1 to 9
    lcdTinyOutPoint();
    lcdTinyOutDigit(num%10);     // ones digit 1 to 9
  }else if(XaxisResolution == 2){//shown as 0.00  0.09  0.99  9.99 
    lcdTinyOutDigit(num/100);    // hundreds digit 1 to 9
   	num = num%100;               // 0 to 99
    lcdTinyOutPoint();
    lcdTinyOutDigit(num/10);     // tens digit 1 to 9
    lcdTinyOutDigit(num%10);     // ones digit 1 to 9
  }else if(XaxisResolution == 3){// shown as .000  .009  .099  .999
    lcdTinyOutDigit(num/100);    // hundreds digit 1 to 9
    lcdTinyOutPoint();
   	num = num%100;               // 0 to 99
    lcdTinyOutDigit(num/10);     // tens digit 1 to 9
    lcdTinyOutDigit(num%10);     // ones digit 1 to 9
  }
}

//-----------------------LCD_PlotXaxis-----------------------
// Draw X axis 
// Input: min max,   
// Output: none
// errors: must be called once, before calling Plot
void LCD_PlotXaxis(unsigned short min, unsigned short max,
  unsigned short resolution, char *label){
  unsigned short i; 
  if(OpenFlag==0){
    return;  // not open
  }
  if((min > max)||(max > 999)){
    return;  // bad parameters
  }
  Xmin = min;
  Xmax = max;
	XaxisResolution = resolution;
	if(XaxisResolution>3){
	  XaxisResolution = 65535;
	}
	i = 0;
	do{
	  Xlabel[i] = *label;
	  label++; i++;
	}
	while(i<14);
	Xlabel[13] = 0; // truncate to 13 characters maximum
	LCD_GoTo(7,6);  // second to last row
  LCD_OutString(Xlabel);
  TinyDown = 2;      // shift down two pixels
  CS2 = 0;   			   // paint left number on left side
  CS1 = 1;           // left enable
  lcdCmd(0xBE);      // Page 7 
  lcdCmd(0x4C);      // byte number = 12
  lcdTinyOutFix(Xmin);
  CS2 = 1;   			   // paint right number on right side
  CS1 = 0;           // right enable
  lcdCmd(0xBE);      // Page 7 
  lcdCmd(0x72);      // byte number = 50
  lcdTinyOutFix(Xmax);
}

unsigned short YaxisResolution;
// Y axis numbers are  -99 to 99
// any other value means no numbers are displayed
// 0  shown as -99     -1    0   1   99
// 1  shown as -9.9  -0.1  0.0 0.1  9.9
// 2  shown as -.99  -.01  .00 .01  .99
short Ymax,Ycenter,Ymin;  // three numbers to put on Yaxis
char Ylabel[10];          // maximum of 8 characters
short MaxY,RangeY;        // used to scale input data
//------lcdTinyOutFix2------------------------
// display 4 character tiny number
// input: num -99 to 99
// output:none
// no error checking
void lcdTinyOutFix2(short snum){ 
unsigned short num;
unsigned short bSign;
  if(snum<0){
    bSign = 1;
    num = -snum;
  } else{
    bSign = 0;
    num = snum;
  }
  if(YaxisResolution == 0){      // shown as  -99     -1    0   1   99
    if(num < 10){
      lcdTinyOutSpace(); 			   // no tens digit
      if(bSign){
        lcdTinyOutMinus();
      } else{
        lcdTinyOutPlus();
      }
    } else{
      if(bSign){
        lcdTinyOutMinus();
      } else{
        lcdTinyOutPlus();
      }
      lcdTinyOutDigit(num/10);   // tens digit 1 to 9
    }
    lcdTinyOutDigit(num%10);     // ones digit 1 to 9
    lcdTinyOutPoint();
  }else if(YaxisResolution == 1){// shown as  -9.9  -0.1  0.0 0.1  9.9
    if(bSign){
      lcdTinyOutMinus();
    } else{
      lcdTinyOutPlus();
    }
    lcdTinyOutDigit(num/10);     // tens digit 1 to 9
    lcdTinyOutPoint();
    lcdTinyOutDigit(num%10);     // ones digit 1 to 9
  }else if(YaxisResolution == 2){// shown as -.99  -.01  .00 .01  .99 
    if(bSign){
      lcdTinyOutMinus();
    } else{
      lcdTinyOutPlus();
    }
    lcdTinyOutPoint();
    lcdTinyOutDigit(num/10);     // tens digit 1 to 9
    lcdTinyOutDigit(num%10);     // ones digit 1 to 9
  }
}
//-----------------------LCD_PlotYaxis-----------------------
// Draw Y axis 
// Input: min, center, max, resolution, label, and number of hash marks
//        min, center, max are the integer part ranging from -99 to +99
//        resolution is 0,1,2 where to put decimal point 
//                      any other value means no numbers are displayed
//  number->    -99    -1    0    1    99     
// 0  shown as -99.   -1.    0.   1.   99.
// 1  shown as -9.9  -0.1   0.0  0.1   9.9
// 2  shown as -.99  -.01   .00  .01   .99
//        label is an ASCII string, up to 8 characters
// Output: none
// errors: must be called once, before calling Plot
void LCD_PlotYaxis(short min, short center, short max,
  unsigned short resolution, unsigned char *label){
  unsigned short i;
  if(OpenFlag==0){
    return;  // not open
  }
  if((min > max)||(center < min)||(center > max)){
    return;  // bad parameters
  }
  if((min < -99)||(max > 99)){
    return;  // bad parameters
  }
  Ymin = min;
  Ycenter = center;
  Ymax = max;
	YaxisResolution = resolution;
	i = 0;
	do{
	  Ylabel[i] = *label;
	  label++; i++;
	}
	while(i<9);
	Ylabel[8] = 0;     // truncate at 8 characters
	
  CS2 = 0;   			   // paint all numbers on left side
  CS1 = 1;           // left enable
	TinyDown = 0;      // shift down no pixels
  lcdCmd(0xB9);      // Page 2 
  lcdCmd(0x48);      // byte number = 8
  lcdTinyOutFix2(Ymax);
	TinyDown = 2;      // shift down two pixels
  lcdCmd(0xBB);      // Page 4 
  lcdCmd(0x48);      // byte number = 8
  lcdTinyOutFix2(Ycenter);
	TinyDown = 3;      // shift down three pixels
  lcdCmd(0xBD);      // Page 6 
  lcdCmd(0x48);      // byte number = 8
  lcdTinyOutFix2(Ymin);

}
//-----------------------LCD_PlotClear-----------------------
// Clear plot window, 
// Input: min max, specify range of Y axis  
// Output: none
// errors: must be called once, before calling Plot
void LCD_PlotClear(short min, short max){
int i,j;
  if(OpenFlag==0){
    return;  // not open
  }
  RangeY = max-min;
  MaxY = max;
  SubCount = PLOT_PIXEL;
  Xvalue = 0;
  CS2 = 0;   			   
  CS1 = 1;           // left enable
  lcdCmd(0xB9);      // Page 2 
  lcdCmd(0x55);      // byte number = 21
  lcdData(0xFF);     // Yaxis line
  lcdData(0x01);     // top hash
  for(i=0;i<41;i++){
    lcdData(0);
  }
  lcdCmd(0xBA);      // Page 3 
  lcdCmd(0x55);      // byte number = 21
  lcdData(0xFF);     // Yaxis line
  lcdData(0x02);     // 75% hash
  for(i=0;i<41;i++){
    lcdData(0);
  }
  lcdCmd(0xBB);      // Page 4 
  lcdCmd(0x55);      // byte number = 21
  lcdData(0xFF);     // Yaxis line
  lcdData(0x10);     // 50% hash
  for(i=0;i<41;i++){
    lcdData(0);
  }
  lcdCmd(0xBC);      // Page 5 
  lcdCmd(0x55);      // byte number = 21
  lcdData(0xFF);     // Yaxis line
  lcdData(0x40);     // 25% hash
  for(i=0;i<41;i++){
    lcdData(0);
  }
  lcdCmd(0xBD);      // Page 6 
  lcdCmd(0x55);      // byte number = 21
  lcdData(0xFF);     // Yaxis line
  for(j=0; j<4; j++){
    for(i=0;i<9;i++){
      lcdData(0x80);  // X axis
    }
    lcdData(0xC0);    // X axis and hash
  }
  lcdData(0x80);      // X axis
  lcdData(0x80);      // X axis
  
  CS2 = 1;   			   
  CS1 = 0;             // right enable
  for(j=0xB9;j<0xBD; j++){
    lcdCmd(j);         // Page 2,3,4,5 
    lcdCmd(0x40);      // byte number = 0
    for(i=0;i<58;i++){
      lcdData(0);
    }
  }
  lcdCmd(0xBD);      // Page 6 
  lcdCmd(0x40);      // byte number = 0
  for(i=0;i<8;i++){
    lcdData(0x80);    // X axis
  }
  lcdData(0xC0);    // X axis and hash
  for(j=0; j<5; j++){
    for(i=0;i<9;i++){
      lcdData(0x80);  // X axis
    }
    lcdData(0xC0);    // X axis and hash
  }
}
//-----------------------LCD_Plot-----------------------
// Plot one data point
// Input: data between min and max  
// Output: none
// errors: must call LCD_PlotClear first
unsigned short pixel;
void LCD_Plot(short data){
  if(OpenFlag==0){
    return;  // not open
  }
  data = 40*(MaxY-data);
  pixel = data/RangeY;  // 0 to 39, truncate down
  if(pixel > 39){
    pixel = 39;
  }
  SubCount--;
  if(SubCount==0){
    Xvalue++;
    if(Xvalue == 100){
      Xvalue = 0;	 // reset back to beginning
    }
    SubCount = PLOT_PIXEL;
  }
}

void main(void) {
  PLL_Init();
  LCD_Init();
  EnableInterrupts;
  LCD_Clear(0);
  LCD_GoTo(1,1);
  LCD_OutString("AGM1264 LCD graphics");
  LCD_GoTo(8,1);
  LCD_OutString("  EE385J.17 Valvano");
  LCD_PlotXaxis(0, 45, 2, "Time (sec)");
  LCD_PlotYaxis(-5, 0, 5, 2, "Voltage");
  LCD_PlotClear(0, 1023);
  for(;;){
  }
}


